@@ -254,7 +254,7 @@ class SphinxThread(QThread):
     doc : str or dict
         A string containing a raw rst text or a dict containing
         the doc string components to be rendered.
-        See spyder.utils.dochelpers.getdoc for description.
+        See spyder_kernels.utils.dochelpers.getdoc for description.
     context : dict
         A dict containing the substitution variables for the
         layout template
@@ -397,7 +397,7 @@ class Help(SpyderPluginWidget):
         self.combo.setMaxCount(self.get_option('max_history_entries'))
         self.combo.addItems( self.load_history() )
         self.combo.setItemText(0, '')
-        self.combo.valid.connect(lambda valid: self.force_refresh())
+        self.combo.valid.connect(lambda valid: self.force_refresh(valid))
 
         # Plain text docstring option
         self.docstring = True
@@ -720,25 +720,21 @@ class Help(SpyderPluginWidget):
 
     def show_rich_text(self, text, collapse=False, img_path=''):
         """Show text in rich mode"""
-        self.visibility_changed(True)
-        self.raise_()
+        self.switch_to_plugin()
         self.switch_to_rich_text()
         context = generate_context(collapse=collapse, img_path=img_path)
         self.render_sphinx_doc(text, context)
 
     def show_plain_text(self, text):
         """Show text in plain mode"""
-        self.visibility_changed(True)
-        self.raise_()
+        self.switch_to_plugin()
         self.switch_to_plain_text()
         self.set_plain_text(text, is_code=False)
 
     @Slot()
     def show_tutorial(self):
         """Show the Spyder tutorial in the Help plugin, opening it if needed"""
-        if not self.dockwidget.isVisible():
-            self.dockwidget.show()
-            self.toggle_view_action.setChecked(True)
+        self.switch_to_plugin()
         tutorial_path = get_module_source_path('spyder.utils.help')
         tutorial = osp.join(tutorial_path, 'tutorial.rst')
         text = open(tutorial).read()
@@ -754,11 +750,12 @@ class Help(SpyderPluginWidget):
             self.rich_text.webview.load(QUrl(url))
 
     #------ Public API ---------------------------------------------------------
-    def force_refresh(self):
-        if self.source_is_console():
-            self.set_object_text(None, force_refresh=True)
-        elif self._last_editor_doc is not None:
-            self.set_editor_doc(self._last_editor_doc, force_refresh=True)
+    def force_refresh(self, valid=True):
+        if valid:
+            if self.source_is_console():
+                self.set_object_text(None, force_refresh=True)
+            elif self._last_editor_doc is not None:
+                self.set_editor_doc(self._last_editor_doc, force_refresh=True)
 
     def set_object_text(self, text, force_refresh=False, ignore_unknown=False):
         """Set object analyzed by Help"""
@@ -819,8 +816,7 @@ class Help(SpyderPluginWidget):
                 if (self.console.dockwidget not in dockwidgets and
                         self.main.ipyconsole is not None and
                         self.main.ipyconsole.dockwidget not in dockwidgets):
-                    self.dockwidget.show()
-                    self.dockwidget.raise_()
+                    self.switch_to_plugin()
         self._last_texts[index] = text
 
     def load_history(self, obj=None):
@@ -834,9 +830,15 @@ class Help(SpyderPluginWidget):
 
     def save_history(self):
         """Save history to a text file in user home directory"""
-        open(self.LOG_PATH, 'w').write("\n".join( \
-                [to_text_string(self.combo.itemText(index))
-                 for index in range(self.combo.count())] ))
+        # Don't fail when saving search history to disk
+        # See issues 8878 and 6864
+        try:
+            search_history = [to_text_string(self.combo.itemText(index))
+                              for index in range(self.combo.count())]
+            search_history = '\n'.join(search_history)
+            open(self.LOG_PATH, 'w').write(search_history)
+        except (UnicodeEncodeError, UnicodeDecodeError, EnvironmentError):
+            pass
 
     @Slot(bool)
     def toggle_plain_text(self, checked):
