@@ -24,6 +24,7 @@ import sys
 
 PY2 = sys.version[0] == '2'
 PY3 = sys.version[0] == '3'
+PY36_OR_MORE = sys.version_info[0] >= 3 and sys.version_info[1] >= 6
 
 #==============================================================================
 # Data types
@@ -79,20 +80,19 @@ else:
     import reprlib
     import queue as Queue
 
+
 #==============================================================================
 # Strings
 #==============================================================================
-if PY2:
-    # Python 2
-    import codecs
-    def u(obj):
-        """Make unicode object"""
-        return codecs.unicode_escape_decode(obj)[0]
-else:
-    # Python 3
-    def u(obj):
-        """Return string as it is"""
-        return obj
+def is_type_text_string(obj):
+    """Return True if `obj` is type text string, False if it is anything else,
+    like an instance of a class that extends the basestring class."""
+    if PY2:
+        # Python 2
+        return type(obj) in [str, unicode]
+    else:
+        # Python 3
+        return type(obj) in [str, bytes]
 
 def is_text_string(obj):
     """Return True if `obj` is a text string, False if it is anything else,
@@ -130,6 +130,8 @@ def is_unicode(obj):
 def to_text_string(obj, encoding=None):
     """Convert `obj` to (unicode) text string"""
     if PY2:
+        if isinstance(obj, unicode):
+            return obj
         # Python 2
         if encoding is None:
             return unicode(obj)
