@@ -192,17 +192,23 @@ class ReadOnlyCollectionsModel(QAbstractTableModel):
             fetch_more = False
         else:
             fetch_more = True
-        
-        if self.remote:
-            sizes = [ data[self.keys[index]]['size'] 
-                      for index in range(start, stop) ]
-            types = [ data[self.keys[index]]['type']
-                      for index in range(start, stop) ]
-        else:
-            sizes = [ get_size(data[self.keys[index]])
-                      for index in range(start, stop) ]
-            types = [ get_human_readable_type(data[self.keys[index]])
-                      for index in range(start, stop) ]
+
+        # Ignore pandas warnings that certain attributes are deprecated
+        # and will be removed, since they will only be accessed if they exist.
+        with warnings.catch_warnings():
+            warnings.filterwarnings(
+                "ignore", message=(r"^\w+\.\w+ is deprecated and "
+                                   "will be removed in a future version"))
+            if self.remote:
+                sizes = [data[self.keys[index]]['size']
+                         for index in range(start, stop)]
+                types = [data[self.keys[index]]['type']
+                         for index in range(start, stop)]
+            else:
+                sizes = [get_size(data[self.keys[index]])
+                         for index in range(start, stop)]
+                types = [get_human_readable_type(data[self.keys[index]])
+                         for index in range(start, stop)]
 
         if fetch_more:
             self.sizes = self.sizes + sizes
@@ -572,16 +578,29 @@ class CollectionsDelegate(QItemDelegate):
             value = data['editor'].get_value()
             conv_func = data.get('conv', lambda v: v)
             self.set_value(index, conv_func(value))
-        self._editors.pop(editor_id)
+        # This is needed to avoid the problem reported on
+        # issue 8557
+        try:
+            self._editors.pop(editor_id)
+        except KeyError:
+            pass
         self.free_memory()
         
     def editor_rejected(self, editor_id):
-        self._editors.pop(editor_id)
+        # This is needed to avoid the problem reported on
+        # issue 8557
+        try:
+            self._editors.pop(editor_id)
+        except KeyError:
+            pass
         self.free_memory()
 
     def free_memory(self):
         """Free memory after closing an editor."""
-        self.sig_free_memory.emit()
+        try:
+            self.sig_free_memory.emit()
+        except RuntimeError:
+            pass
 
     def commitAndCloseEditor(self):
         """Overriding method commitAndCloseEditor"""
@@ -1128,7 +1147,13 @@ class BaseTableView(QTableView):
             elif isinstance(obj, (DataFrame, Series)) \
               and DataFrame is not FakeObject:
                 output = io.StringIO()
-                obj.to_csv(output, sep='\t', index=True, header=True)
+                try:
+                    obj.to_csv(output, sep='\t', index=True, header=True)
+                except Exception:
+                    QMessageBox.warning(self, _("Warning"),
+                                        _("It was not possible to copy "
+                                          "this dataframe"))
+                    return
                 if PY3:
                     obj = output.getvalue()
                 else:
@@ -1144,6 +1169,9 @@ class BaseTableView(QTableView):
     def import_from_string(self, text, title=None):
         """Import data from string"""
         data = self.model.get_data()
+        # Check if data is a dict
+        if not hasattr(data, "keys"):
+            return
         editor = ImportWizard(self, text, title=title,
                               contents_title=_("Clipboard contents"),
                               varname=fix_reference_name("data",
@@ -1351,6 +1379,12 @@ class CollectionsEditor(QDialog):
                 readonly = True
                 self.data_copy = data
             datalen = len(get_object_attrs(data))
+
+        # If the copy has a different type, then do not allow editing, because
+        # this would change the type after saving; cf. issue #6936
+        if type(self.data_copy) != type(data):
+            readonly = True
+
         self.widget = CollectionsEditorWidget(self, self.data_copy,
                                               title=title, readonly=readonly,
                                               remote=remote)
