@@ -1,16 +1,24 @@
 # -*- coding: utf-8 -*-
+# -----------------------------------------------------------------------------
+# Copyright (c) 2011-2012 Lambda Foundry, Inc. and PyData Development Team
+# Copyright (c) 2013 Jev Kuznetsov and contributors
+# Copyright (c) 2014- Spyder Project Contributors
 #
-# Copyright © Spyder Project Contributors
-# Licensed under the terms of the New BSD License
-#
-# DataFrameModel is based on the class ArrayModel from array editor
-# and the class DataFrameModel from the pandas project.
-# Present in pandas.sandbox.qtpandas in v0.13.1
-# Copyright (c) 2011-2012, Lambda Foundry, Inc.
-# and PyData Development Team All rights reserved
+# Distributed under the terms of the New BSD License
+# (BSD 3-clause; see NOTICE.txt in the Spyder root directory for details).
+# -----------------------------------------------------------------------------
 
 """
-Pandas DataFrame Editor Dialog
+Pandas DataFrame Editor Dialog.
+
+DataFrameModel is based on the class ArrayModel from array editor
+and the class DataFrameModel from the pandas project.
+Present in pandas.sandbox.qtpandas in v0.13.1.
+
+Originally based on pandas/sandbox/qtpandas.py of the
+`pandas project <https://github.com/pandas-dev/pandas>`_.
+The current version is qtpandas/models/DataFrameModel.py of the
+`QtPandas project <https://github.com/draperjames/qtpandas>`_.
 """
 
 # Third party imports
@@ -18,21 +26,23 @@ from qtpy import API
 from qtpy.compat import from_qvariant, to_qvariant
 from qtpy.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal, Slot
 from qtpy.QtGui import QColor, QCursor
-from qtpy.QtWidgets import (QApplication, QCheckBox, QDialogButtonBox, QDialog,
-                            QGridLayout, QHBoxLayout, QInputDialog, QLineEdit,
-                            QMenu, QMessageBox, QPushButton, QTableView,
-                            QHeaderView)
+from qtpy.QtWidgets import (QApplication, QCheckBox, QDialog, QGridLayout,
+                            QHBoxLayout, QInputDialog, QLineEdit, QMenu,
+                            QMessageBox, QPushButton, QTableView, QHeaderView)
 
 from pandas import DataFrame, DatetimeIndex, Series
+try:
+    from pandas._libs.tslib import OutOfBoundsDatetime
+except ImportError:  # For pandas version < 0.20
+    from pandas.tslib import OutOfBoundsDatetime
 import numpy as np
 
 # Local imports
 from spyder.config.base import _
 from spyder.config.fonts import DEFAULT_SMALL_DELTA
 from spyder.config.gui import get_font, config_shortcut
-from spyder.py3compat import (io, is_text_string, PY2, to_text_string,
-                              TEXT_TYPES)
-from spyder.utils import encoding
+from spyder.py3compat import (io, is_text_string, is_type_text_string, PY2,
+                              to_text_string)
 from spyder.utils import icon_manager as ima
 from spyder.utils.qthelpers import (add_actions, create_action,
                                     keybinding, qapplication)
@@ -42,7 +52,7 @@ from spyder.widgets.variableexplorer.arrayeditor import get_idx_rect
 REAL_NUMBER_TYPES = (float, int, np.int64, np.int32)
 COMPLEX_NUMBER_TYPES = (complex, np.complex64, np.complex128)
 # Used to convert bool intrance to false since bool('False') will return True
-_bool_false = ['false', '0']
+_bool_false = ['false', 'f', '0', '0.', '0.0', ' ']
 
 # Default format for data frames with floats
 DEFAULT_FORMAT = '%.6g'
@@ -95,6 +105,7 @@ class DataFrameModel(QAbstractTableModel):
         self.df_header = dataFrame.columns.tolist()
         self._format = format
         self.complex_intran = None
+        self.display_error_idxs = []
         
         self.total_rows = self.df.shape[0]
         self.total_cols = self.df.shape[1]
@@ -191,31 +202,26 @@ class DataFrameModel(QAbstractTableModel):
             if section == 0:
                 return 'Index'
             elif section == 1 and PY2:
+                header = self.df_header[0]
                 # Get rid of possible BOM utf-8 data present at the
                 # beginning of a file, which gets attached to the first
                 # column header when headers are present in the first
                 # row.
                 # Fixes Issue 2514
                 try:
-                    header = to_text_string(self.df_header[0],
-                                            encoding='utf-8-sig')
+                    header = to_text_string(header, encoding='utf-8-sig')
                 except:
-                    header = to_text_string(self.df_header[0])
+                    # Don't perform any conversion on strings because it
+                    # leads to differences between the data present in
+                    # the dataframe and what is shown by Spyder
+                    if not is_type_text_string(header):
+                        header = to_text_string(header)
                 return to_qvariant(header)
-            elif isinstance(self.df_header[section-1], TEXT_TYPES):
-                # Get the proper encoding of the text in the header.
-                # Fixes Issue 3896
-                if not PY2:
-                    try:
-                        header = self.df_header[section-1].encode('utf-8')
-                        coding = 'utf-8-sig'
-                    except:
-                        header = self.df_header[section-1].encode('utf-8')
-                        coding = encoding.get_coding(header)
-                else:
-                    header = self.df_header[section-1]
-                    coding = encoding.get_coding(header)
-                return to_qvariant(to_text_string(header, encoding=coding))
+            elif is_type_text_string(self.df_header[section-1]):
+                # Don't perform any conversion on strings because it
+                # leads to differences between the data present in
+                # the dataframe and what is shown by Spyder
+                return to_qvariant(self.df_header[section-1])
             else:
                 return to_qvariant(to_text_string(self.df_header[section-1]))
         else:
@@ -258,6 +264,8 @@ class DataFrameModel(QAbstractTableModel):
         # handling, so fallback uses iloc
         try:
             value = self.df.iat[row, column]
+        except OutOfBoundsDatetime:
+            value = self.df.iloc[:, column].astype(str).iat[row]
         except:
             value = self.df.iloc[row, column]
         return value
@@ -274,7 +282,15 @@ class DataFrameModel(QAbstractTableModel):
             column = index.column()
             row = index.row()
             if column == 0:
-                return to_qvariant(to_text_string(self.df_index[row]))
+                df_idx = self.df_index[row]
+                if is_type_text_string(df_idx):
+                    # Don't perform any conversion on strings
+                    # because it leads to differences between
+                    # the data present in the dataframe and
+                    # what is shown by Spyder
+                    return df_idx
+                else:
+                    return to_qvariant(to_text_string(df_idx))
             else:
                 value = self.get_value(row, column-1)
                 if isinstance(value, float):
@@ -284,15 +300,26 @@ class DataFrameModel(QAbstractTableModel):
                         # may happen if format = '%d' and value = NaN;
                         # see issue 4139
                         return to_qvariant(DEFAULT_FORMAT % value)
+                elif is_type_text_string(value):
+                    # Don't perform any conversion on strings
+                    # because it leads to differences between
+                    # the data present in the dataframe and
+                    # what is shown by Spyder
+                    return value
                 else:
                     try:
                         return to_qvariant(to_text_string(value))
-                    except UnicodeDecodeError:
-                        return to_qvariant(encoding.to_unicode(value))
+                    except Exception:
+                        self.display_error_idxs.append(index)
+                        return u'Display Error!'
         elif role == Qt.BackgroundColorRole:
             return to_qvariant(self.get_bgcolor(index))
         elif role == Qt.FontRole:
             return to_qvariant(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
+        elif role == Qt.ToolTipRole:
+            if index in self.display_error_idxs:
+                return _("It is not possible to display this value because\n"
+                         "an error ocurred while trying to do it")
         return to_qvariant()
 
     def sort(self, column, order=Qt.AscendingOrder):
@@ -347,6 +374,8 @@ class DataFrameModel(QAbstractTableModel):
         column = index.column()
         row = index.row()
 
+        if index in self.display_error_idxs:
+            return False
         if change_type is not None:
             try:
                 value = self.data(index, role=Qt.DisplayRole)
@@ -359,23 +388,24 @@ class DataFrameModel(QAbstractTableModel):
         else:
             val = from_qvariant(value, str)
             current_value = self.get_value(row, column-1)
-            if isinstance(current_value, bool):
+            if isinstance(current_value, (bool, np.bool_)):
                 val = bool_false_check(val)
-            supported_types = (bool,) + REAL_NUMBER_TYPES + COMPLEX_NUMBER_TYPES
-            if (isinstance(current_value, supported_types) or 
+            supported_types = (bool, np.bool_) + REAL_NUMBER_TYPES
+            if (isinstance(current_value, supported_types) or
                     is_text_string(current_value)):
                 try:
                     self.df.iloc[row, column-1] = current_value.__class__(val)
-                except ValueError as e:
+                except (ValueError, OverflowError) as e:
                     QMessageBox.critical(self.dialog, "Error",
-                                         "Value error: %s" % str(e))
+                                         str(type(e).__name__) + ": " + str(e))
                     return False
             else:
                 QMessageBox.critical(self.dialog, "Error",
-                                     "The type of the cell is not a supported "
-                                     "type")
+                                     "Editing dtype {0!s} not yet supported."
+                                     .format(type(current_value).__name__))
                 return False
         self.max_min_col_update()
+        self.dataChanged.emit(index, index)
         return True
 
     def get_data(self):
@@ -384,10 +414,16 @@ class DataFrameModel(QAbstractTableModel):
 
     def rowCount(self, index=QModelIndex()):
         """DataFrame row number"""
-        if self.total_rows <= self.rows_loaded:
-            return self.total_rows
-        else:
-            return self.rows_loaded
+        # Avoid a "Qt exception in virtual methods" generated in our
+        # tests on Windows/Python 3.7
+        # See PR 8910
+        try:
+            if self.total_rows <= self.rows_loaded:
+                return self.total_rows
+            else:
+                return self.rows_loaded
+        except AttributeError:
+            return 0
 
     def can_fetch_more(self, rows=False, columns=False):
         if rows:
@@ -419,13 +455,19 @@ class DataFrameModel(QAbstractTableModel):
 
     def columnCount(self, index=QModelIndex()):
         """DataFrame column number"""
-        # This is done to implement series
-        if len(self.df.shape) == 1:
-            return 2
-        elif self.total_cols <= self.cols_loaded:
-            return self.total_cols + 1
-        else:
-            return self.cols_loaded + 1
+        # Avoid a "Qt exception in virtual methods" generated in our
+        # tests on Windows/Python 3.7
+        # See PR 8910
+        try:
+            # This is done to implement series
+            if len(self.df.shape) == 1:
+                return 2
+            elif self.total_cols <= self.cols_loaded:
+                return self.total_cols + 1
+            else:
+                return self.cols_loaded + 1
+        except AttributeError:
+            return 0
 
     def reset(self):
         self.beginResetModel()
@@ -435,7 +477,7 @@ class DataFrameModel(QAbstractTableModel):
 class FrozenTableView(QTableView):
     """This class implements a table with its first column frozen
     For more information please see:
-    http://doc.qt.io/qt-5/qtwidgets-itemviews-frozencolumn-example.html"""
+    https://doc.qt.io/qt-5/qtwidgets-itemviews-frozencolumn-example.html"""
     def __init__(self, parent):
         """Constructor."""
         QTableView.__init__(self, parent)
@@ -685,6 +727,7 @@ class DataFrameEditor(QDialog):
         self.resize(600, 500)
 
         self.dataModel = DataFrameModel(data, parent=self)
+        self.dataModel.dataChanged.connect(self.save_and_close_enable)
         self.dataTable = DataFrameView(self, self.dataModel)
 
         self.layout.addWidget(self.dataTable)
@@ -716,15 +759,29 @@ class DataFrameEditor(QDialog):
         btn_layout.addWidget(self.bgcolor_global)
 
         btn_layout.addStretch()
-        bbox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
-        bbox.accepted.connect(self.accept)
-        bbox.rejected.connect(self.reject)
-        btn_layout.addWidget(bbox)
+
+        self.btn_save_and_close = QPushButton(_('Save and Close'))
+        self.btn_save_and_close.setDisabled(True)
+        self.btn_save_and_close.clicked.connect(self.accept)
+        btn_layout.addWidget(self.btn_save_and_close)
+
+        self.btn_close = QPushButton(_('Close'))
+        self.btn_close.setAutoDefault(True)
+        self.btn_close.setDefault(True)
+        self.btn_close.clicked.connect(self.reject)
+        btn_layout.addWidget(self.btn_close)
 
         self.layout.addLayout(btn_layout, 2, 0)
 
         return True
 
+    @Slot(QModelIndex, QModelIndex)
+    def save_and_close_enable(self, top_left, bottom_right):
+        """Handle the data change event to enable the save and close button."""
+        self.btn_save_and_close.setEnabled(True)
+        self.btn_save_and_close.setAutoDefault(True)
+        self.btn_save_and_close.setDefault(True)
+
     def change_bgcolor_enable(self, state):
         """
         This is implementet so column min/max is only active when bgcolor is
