@@ -167,9 +167,6 @@ class PylintWidget(QWidget):
                 pass
 
         self.filecombo = PythonModulesComboBox(self)
-        if self.rdata:
-            self.remove_obsolete_items()
-            self.filecombo.addItems(self.get_filenames())
         
         self.start_button = create_toolbutton(self, icon=ima.icon('run'),
                                     text=_("Analyze"),
@@ -218,7 +215,14 @@ class PylintWidget(QWidget):
         self.process = None
         self.set_running_state(False)
         self.show_data()
-        
+
+        if self.rdata:
+            self.remove_obsolete_items()
+            self.filecombo.addItems(self.get_filenames())
+            self.start_button.setEnabled(self.filecombo.is_valid())
+        else:
+            self.start_button.setEnabled(False)
+
     def analyze(self, filename):
         filename = to_text_string(filename) # filename is a QString instance
         self.kill_if_running()
@@ -355,7 +359,7 @@ class PylintWidget(QWidget):
                 module = line[len(txt_module):]
                 continue
             # Supporting option include-ids: ('R3873:' instead of 'R:')
-            if not re.match('^[CRWE]+([0-9]{4})?:', line):
+            if not re.match(r'^[CRWE]+([0-9]{4})?:', line):
                 continue
             i1 = line.find(':')
             if i1 == -1:
